@@ -51,7 +51,7 @@ def delete_obj(obj):
 def memory_used(func, *args, **kwargs):
     """Compute memory usage of func."""
     if memory_usage is None:
-        return np.NaN
+        return np.nan
 
     gc.collect()
     mem_use = memory_usage((func, args, kwargs), interval=.001)
